# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C) 2015 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Note: YOUR-ACCESSKEYID, YOUR-SECRETACCESSKEY, my-bucketname and my-prefixname
# are dummy values, please replace them with original values.

from minio import Minio

client = Minio('s3.amazonaws.com',
               access_key='YOUR-ACCESSKEYID',
               secret_key='YOUR-SECRETACCESSKEY')

# List all object paths in bucket that begin with my-prefixname.
objects = client.list_objects('my-bucketname', prefix='my-prefixname',
                              recursive=True)
for obj in objects:
    print(obj.bucket_name, obj.object_name.encode('utf-8'), obj.last_modified,
          obj.etag, obj.size, obj.content_type)

# List all object paths in bucket that begin with my-prefixname using
# V2 listing API.
objects = client.list_objects_v2('my-bucketname', prefix='my-prefixname',
                              recursive=True)
for obj in objects:
    print(obj.bucket_name, obj.object_name.encode('utf-8'), obj.last_modified,
          obj.etag, obj.size, obj.content_type)



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C)
# 2020 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
import json
from unittest import TestCase
from minio.credentials.aws_iam import IamEc2MetaData
from nose.tools import eq_

class CredListResponse(object):
    status = 200
    data = b'test-s3-full-access-for-minio-ec2'

class CredsResponse(object):
    status = 200
    data = json.dumps({
            "Code" : 'Success',
            "Type" : 'AWS-HMAC',
            "AccessKeyId" : 'accessKey',
            "SecretAccessKey" : 'secret',
            "Token" : 'token',
            "Expiration" : '2014-12-16T01:51:37Z',
            "LastUpdated" : '2009-11-23T0:00:00Z'
        })

class TestIamEc2MetaData(TestCase):
    @mock.patch('urllib3.PoolManager.urlopen')
    def test_iam(self, mock_connection):
        # get provider
        mock_connection.side_effect = [CredListResponse(), CredsResponse()]
        provider = IamEc2MetaData()
        # retrieve credentials
        creds = provider.retrieve()
        eq_(creds.access_key, "accessKey")
        eq_(creds.secret_key, "secret")
        eq_(creds.session_token, "token")



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C) 2015 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from unittest import TestCase

import mock
from nose.tools import eq_, raises

from minio import Minio
from minio.api import _DEFAULT_USER_AGENT
from minio.error import NoSuchBucket
from tests.unit.minio_mocks import (
    MockConnection,
    MockResponse
)

class GetBucketPolicyTest(TestCase):
    @mock.patch('urllib3.PoolManager')
    @raises(NoSuchBucket)
    def test_get_policy_for_non_existent_bucket(self, mock_connection):
        mock_server = MockConnection()
        mock_connection.return_value = mock_server
        bucket_name = 'non-existent-bucket'
        mock_server.mock_add_request(
            MockResponse(
                'GET',
                'https://localhost:9000/' + bucket_name + '/?policy=',
                {'User-Agent': _DEFAULT_USER_AGENT},
                404,
            )
        )
        client = Minio('localhost:9000')
        client.get_bucket_policy(bucket_name)

    @mock.patch('urllib3.PoolManager')
    def test_get_policy_for_existent_bucket(self, mock_connection):
        mock_data = {
            "Version":"2012-10-17",
            "Statement":[
                {
                "Sid":"",
                "Effect":"Allow",
                "Principal":{"AWS":"*"},
                "Action":"s3:GetBucketLocation",
                "Resource":"arn:aws:s3:::test-bucket"
                },
                {
                "Sid":"",
                "Effect":"Allow",
                "Principal":{"AWS":"*"},
                "Action":"s3:ListBucket",
                "Resource":"arn:aws:s3:::test-bucket"
                },
                {
                "Sid":"",
                "Effect":"Allow",
                "Principal":{"AWS":"*"},
                "Action":"s3:GetObject",
                "Resource":"arn:aws:s3:::test-bucket/*"
                }
            ]
        }
        mock_server = MockConnection()
        mock_connection.return_value = mock_server
        bucket_name = 'test-bucket'
        mock_server.mock_add_request(
            MockResponse(
                'GET',
                'https://localhost:9000/' + bucket_name + '/?policy=',
                {'User-Agent': _DEFAULT_USER_AGENT},
                200,
                content=mock_data
            )
        )
        client = Minio('localhost:9000')
        response = client.get_bucket_policy(bucket_name)
        eq_(response, mock_data)



import base64
from io import BytesIO
import hashlib

from minio.api import Minio

AWSAccessKeyId = ''
AWSSecretKey = ''

STORAGE_ENDPOINT = 's3.amazonaws.com'
STORAGE_BUCKET = ''


def main():
    content = BytesIO(b'Hello again')

    key = b'32byteslongsecretkeymustprovided'
    encryption_key = base64.b64encode(key).decode()
    encryption_key_md5 = base64.b64encode(hashlib.md5(key).digest()).decode()

    minio = Minio(STORAGE_ENDPOINT, access_key=AWSAccessKeyId, secret_key=AWSSecretKey)

    # Put object with special headers which encrypt object in S3 with provided key
    minio.put_object(STORAGE_BUCKET, 'test_crypt.txt', content, content.getbuffer().nbytes,
                     metadata={
                         'x-amz-server-side-encryption-customer-algorithm': 'AES256',
                         'x-amz-server-side-encryption-customer-key': encryption_key,
                         'x-amz-server-side-encryption-customer-key-MD5': encryption_key_md5
                     })

    # Get decrypted object with same headers
    obj = minio.get_object(STORAGE_BUCKET, 'test_crypt1.txt', request_headers={
        'x-amz-server-side-encryption-customer-algorithm': 'AES256',
        'x-amz-server-side-encryption-customer-key': encryption_key,
        'x-amz-server-side-encryption-customer-key-MD5': encryption_key_md5
    })

    print(obj.read())

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage,
# (C) 2015, 2016 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from nose.tools import eq_, raises
from unittest import TestCase

from minio.helpers import optimal_part_info, MAX_MULTIPART_OBJECT_SIZE, MIN_PART_SIZE
from minio.error import InvalidArgumentError

class TraceTest(TestCase):
    @raises(InvalidArgumentError)
    def test_input_size_wrong_default(self):
        optimal_part_info(MAX_MULTIPART_OBJECT_SIZE + 1, MIN_PART_SIZE)

    def test_configured_input_size_valid_maximum(self):
        total_parts_count, part_size, last_part_size = optimal_part_info(MAX_MULTIPART_OBJECT_SIZE, 1024*1024*1000)
        eq_(total_parts_count, 5243)
        eq_(part_size, 1048576000)
        eq_(last_part_size, 922746880)

    def test_configured_input_size_valid(self):
        total_parts_count, part_size, last_part_size = optimal_part_info(MAX_MULTIPART_OBJECT_SIZE/1024, 64*1024*1024)
        eq_(total_parts_count, 80)
        eq_(part_size, 67108864)
        eq_(last_part_size, 67108864)

    def test_configured_input_size_is_special_value(self):
        total_parts_count, part_size, last_part_size = optimal_part_info(-1, 1024*1024*1000)
        eq_(total_parts_count, 5243)
        eq_(part_size, 1048576000)
        eq_(last_part_size, 922746880)

    def test_input_size_valid_maximum_default(self):
        total_parts_count, part_size, last_part_size = optimal_part_info(MAX_MULTIPART_OBJECT_SIZE, MIN_PART_SIZE)
        eq_(total_parts_count, 9987)
        eq_(part_size, 550502400)
        eq_(last_part_size, 241172480)

    def test_input_size_valid_default(self):
        total_parts_count, part_size, last_part_size = optimal_part_info(MAX_MULTIPART_OBJECT_SIZE/1024, MIN_PART_SIZE)
        eq_(total_parts_count, 1024)
        eq_(part_size, 5242880)
        eq_(last_part_size, 5242880)

    def test_input_size_is_special_value_default(self):
        total_parts_count, part_size, last_part_size = optimal_part_info(-1, MIN_PART_SIZE)
        eq_(total_parts_count, 9987)
        eq_(part_size, 550502400)
        eq_(last_part_size, 241172480)



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C) 2015 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import TestCase
from nose.tools import raises

from minio import Minio

class PutObjectTest(TestCase):
    @raises(TypeError)
    def test_object_is_string(self):
        client = Minio('localhost:9000')
        client.put_object('hello', 1234, 1, iter([1, 2, 3]))

    @raises(ValueError)
    def test_object_is_not_empty_string(self):
        client = Minio('localhost:9000')
        client.put_object('hello', ' \t \n ', 1, iter([1, 2, 3]))

    @raises(TypeError)
    def test_length_is_string(self):
        client = Minio('localhost:9000')
        client.put_object('hello', 1234, '1', iter([1, 2, 3]))

    @raises(ValueError)
    def test_length_is_not_empty_string(self):
        client = Minio('localhost:9000')
        client.put_object('hello', ' \t \n ', -1, iter([1, 2, 3]))



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage,
# (C) 2015, 2016, 2017 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
minio - MinIO Python Library for Amazon S3 Compatible Cloud Storage
~~~~~~~~~~~~~~~~~~~~~

   >>> import minio
   >>> minio = Minio('https://s3.amazonaws.com')
   >>> for bucket in minio.list_buckets():
   ...     print(bucket.name)

:copyright: (c) 2015, 2016, 2017 by MinIO, Inc.
:license: Apache 2.0, see LICENSE for more details.
"""

__title__ = 'minio-py'
__author__ = 'MinIO, Inc.'
__version__ = '5.0.8'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2015, 2016, 2017, 2018, 2019 MinIO, Inc.'

from .api import Minio
from .error import ResponseError
from .post_policy import PostPolicy
from .copy_conditions import CopyConditions
from .definitions import Bucket, Object



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C)
# 2020 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from unittest import TestCase

from minio.credentials.env_aws import EnvAWS
from minio.credentials.credentials import Value
from nose.tools import eq_

class EnvAWSTest(TestCase):

    def test_env_aws_retrieve(self):
        # clear environement
        os.environ.clear()
        # set environment variables
        os.environ["AWS_ACCESS_KEY_ID"] = "access"
        os.environ["AWS_SECRET_ACCESS_KEY"] = "secret"
        os.environ["AWS_SESSION_TOKEN"] = "token"
        # init new env_aws provider
        provider = EnvAWS()
        # assert expired true for newly created provider
        eq_(provider.is_expired(), True)
        # retrieve provider credentials
        creds = provider.retrieve()
        # assert expected data
        expected_creds = Value(
            access_key="access",
            secret_key="secret",
            session_token="token"
        )
        eq_(creds.access_key, expected_creds.access_key)
        eq_(creds.secret_key, expected_creds.secret_key)
        eq_(creds.session_token, expected_creds.session_token)
        # assert expired true for retrieved credentials
        eq_(provider.is_expired(), False)

    def test_env_aws_retrieve_no_token(self):
        # clear environement
        os.environ.clear()
        # set environment variables
        os.environ["AWS_ACCESS_KEY_ID"] = "access"
        os.environ["AWS_SECRET_ACCESS_KEY"] = "secret"
        # Init new env_aws provider
        provider = EnvAWS()
        # assert expired true for newly created provider
        eq_(provider.is_expired(), True)
        # retrieve rpovider credentials
        creds = provider.retrieve()
        # assert expected data
        expected_creds = Value(
            access_key="access",
            secret_key="secret",
            session_token=None
        )
        eq_(creds.access_key, expected_creds.access_key)
        eq_(creds.secret_key, expected_creds.secret_key)
        eq_(creds.session_token, expected_creds.session_token)
        # assert expired true for retrieved credentials
        eq_(provider.is_expired(), False)



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage,
# (C) 2015, 2016 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import TestCase
from nose.tools import raises

from minio import Minio
from minio.error import InvalidBucketError
from minio.copy_conditions import CopyConditions

class CopyObjectTest(TestCase):
    @raises(TypeError)
    def test_object_is_string(self):
        client = Minio('localhost:9000')
        client.copy_object('hello', 12, 12)

    @raises(ValueError)
    def test_object_is_not_empty_string(self):
        client = Minio('localhost:9000')
        client.copy_object('hello', ' \t \n ', '')

    @raises(InvalidBucketError)
    def test_length_is_string(self):
        client = Minio('localhost:9000')
        client.copy_object('..hello', '1', '/testbucket/object')

class CopyConditionTest(TestCase):
    @raises(ValueError)
    def test_match_etag_is_not_empty(self):
        conds = CopyConditions()
        conds.set_match_etag('')

    @raises(ValueError)
    def test_match_etag_is_not_empty_except(self):
        conds = CopyConditions()
        conds.set_match_etag_except('')

    @raises(AttributeError)
    def test_unmodified_since(self):
        conds = CopyConditions()
        conds.set_unmodified_since('')

    @raises(AttributeError)
    def test_modified_since(self):
        conds = CopyConditions()
        conds.set_modified_since('')



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage,
# (C) 2015, 2016 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from nose.tools import eq_
from unittest import TestCase

from minio import Minio
from minio.api import _DEFAULT_USER_AGENT

from .minio_mocks import MockResponse, MockConnection

class ListPartsTest(TestCase):
    @mock.patch('urllib3.PoolManager')
    def test_empty_list_parts_works(self, mock_connection):
        mock_data = '''<?xml version="1.0"?>
                       <ListPartsResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
                         <Bucket>bucket</Bucket>
                         <Key>go1.4.2</Key>
                         <UploadId>ntWSjzBytPT2xKLaMRonzXncsO10EH4Fc-Iq2-4hG-ulRYB</UploadId>
                         <Initiator>
                           <ID>minio</ID>
                           <DisplayName>minio</DisplayName>
                         </Initiator>
                         <Owner>
                           <ID>minio</ID>
                           <DisplayName>minio</DisplayName>
                         </Owner>
                         <StorageClass>STANDARD</StorageClass>
                         <PartNumberMarker>0</PartNumberMarker>
                         <NextPartNumberMarker>0</NextPartNumberMarker>
                         <MaxParts>1000</MaxParts>
                         <IsTruncated>false</IsTruncated>
                       </ListPartsResult>
                    '''
        mock_server = MockConnection()
        mock_connection.return_value = mock_server
        mock_server.mock_add_request(
            MockResponse('GET',
                         'https://localhost:9000/bucket/key?uploadId=upload_id',
                         {'User-Agent': _DEFAULT_USER_AGENT}, 200, content=mock_data))

        client = Minio('localhost:9000')
        part_iter = client._list_object_parts('bucket', 'key', 'upload_id')
        parts = []
        for part in part_iter:
            parts.append(part)
        eq_(0, len(parts))

    @mock.patch('urllib3.PoolManager')
    def test_list_object_parts_works(self, mock_connection):
        mock_data = '''<?xml version="1.0"?>
                       <ListPartsResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
                         <Bucket>bucket</Bucket>
                         <Key>go1.4.2</Key>
                         <UploadId>ntWSjzBytPT2xKLaMRonzXncsO10EH4Fc-Iq2-4hG-ulRYB</UploadId>
                         <Initiator>
                           <ID>minio</ID>
                           <DisplayName>minio</DisplayName>
                         </Initiator>
                         <Owner>
                           <ID>minio</ID>
                           <DisplayName>minio</DisplayName>
                         </Owner>
                         <StorageClass>STANDARD</StorageClass>
                         <PartNumberMarker>0</PartNumberMarker>
                         <NextPartNumberMarker>0</NextPartNumberMarker>
                         <MaxParts>1000</MaxParts>
                         <IsTruncated>false</IsTruncated>
                         <Part>
                           <PartNumber>1</PartNumber>
                           <ETag>79b281060d337b9b2b84ccf390adcf74</ETag>
                           <LastModified>2015-06-03T03:12:34.756Z</LastModified>
                           <Size>5242880</Size>
                         </Part>
                         <Part>
                           <PartNumber>2</PartNumber>
                           <ETag>79b281060d337b9b2b84ccf390adcf74</ETag>
                           <LastModified>2015-06-03T03:12:34.756Z</LastModified>
                           <Size>5242880</Size>
                         </Part>
                       </ListPartsResult>
                    '''
        mock_server = MockConnection()
        mock_connection.return_value = mock_server
        mock_server.mock_add_request(MockResponse('GET',
                                                  'https://localhost:9000/bucket/key?uploadId=upload_id',
                                                  {'User-Agent': _DEFAULT_USER_AGENT}, 200,
                                                  content=mock_data))
        client = Minio('localhost:9000')
        part_iter = client._list_object_parts('bucket', 'key', 'upload_id')

        parts = []
        for part in part_iter:
            parts.append(part)
        eq_(2, len(parts))

    @mock.patch('urllib3.PoolManager')
    def test_list_objects_works(self, mock_connection):
        mock_data1 = '''<?xml version="1.0"?>
                        <ListPartsResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
                          <Bucket>bucket</Bucket>
                          <Key>go1.4.2</Key>
                          <UploadId>ntWSjzBytPT2xKLaMRonzXncsO10EH4Fc-Iq2-4hG-ulRYB</UploadId>
                          <Initiator>
                            <ID>minio</ID>
                            <DisplayName>minio</DisplayName>
                          </Initiator>
                          <Owner>
                            <ID>minio</ID>
                            <DisplayName>minio</DisplayName>
                          </Owner>
                          <StorageClass>STANDARD</StorageClass>
                          <PartNumberMarker>0</PartNumberMarker>
                          <NextPartNumberMarker>2</NextPartNumberMarker>
                          <MaxParts>1000</MaxParts>
                          <IsTruncated>true</IsTruncated>
                          <Part>
                            <PartNumber>1</PartNumber>
                            <ETag>79b281060d337b9b2b84ccf390adcf74</ETag>
                            <LastModified>2015-06-03T03:12:34.756Z</LastModified>
                            <Size>5242880</Size>
                          </Part>
                          <Part>
                            <PartNumber>2</PartNumber>
                            <ETag>79b281060d337b9b2b84ccf390adcf74</ETag>
                            <LastModified>2015-06-03T03:12:34.756Z</LastModified>
                            <Size>5242880</Size>
                          </Part>
                        </ListPartsResult>
                        '''
        mock_data2 = '''<?xml version="1.0"?>
                        <ListPartsResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
                          <Bucket>bucket</Bucket>
                          <Key>go1.4.2</Key>
                          <UploadId>ntWSjzBytPT2xKLaMRonzXncsO10EH4Fc-Iq2-4hG-ulRYB</UploadId>
                          <Initiator>
                            <ID>minio</ID>
                            <DisplayName>minio</DisplayName>
                          </Initiator>
                          <Owner>
                            <ID>minio</ID>
                            <DisplayName>minio</DisplayName>
                          </Owner>
                          <StorageClass>STANDARD</StorageClass>
                          <PartNumberMarker>0</PartNumberMarker>
                          <NextPartNumberMarker>0</NextPartNumberMarker>
                          <MaxParts>1000</MaxParts>
                          <IsTruncated>false</IsTruncated>
                          <Part>
                            <PartNumber>3</PartNumber>
                            <ETag>79b281060d337b9b2b84ccf390adcf74</ETag>
                            <LastModified>2015-06-03T03:12:34.756Z</LastModified>
                            <Size>5242880</Size>
                          </Part>
                          <Part>
                            <PartNumber>4</PartNumber>
                            <ETag>79b281060d337b9b2b84ccf390adcf74</ETag>
                            <LastModified>2015-06-03T03:12:34.756Z</LastModified>
                            <Size>5242880</Size>
                          </Part>
                        </ListPartsResult>
                     '''
        mock_server = MockConnection()
        mock_connection.return_value = mock_server
        mock_server.mock_add_request(
            MockResponse('GET',
                         'https://localhost:9000/bucket/key?uploadId=upload_id',
                         {'User-Agent': _DEFAULT_USER_AGENT}, 200, content=mock_data1))


        client = Minio('localhost:9000')
        part_iter = client._list_object_parts('bucket', 'key', 'upload_id')

        parts = []
        for part in part_iter:
            mock_server.mock_add_request(
                MockResponse('GET',
                             'https://localhost:9000/bucket/key?part-number-marker=2&uploadId=upload_id',
                             {'User-Agent': _DEFAULT_USER_AGENT}, 200, content=mock_data2))
            parts.append(part)
        eq_(4, len(parts))



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C) 2015 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from unittest import TestCase
from nose.tools import raises

from minio import Minio
from minio.api import _DEFAULT_USER_AGENT
from minio.error import InvalidArgumentError

from .minio_mocks import MockResponse, MockConnection

class SetBucketNotificationTest(TestCase):
    @raises(TypeError)
    def test_notification_is_dict_1(self):
        client = Minio('localhost:9000')
        client.set_bucket_notification('my-test-bucket', 'abc')

    @raises(TypeError)
    def test_notification_is_dict_2(self):
        client = Minio('localhost:9000')
        client.set_bucket_notification('my-test-bucket', ['myconfig1'])

    @raises(InvalidArgumentError)
    def test_notification_config_is_nonempty(self):
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {}
        )

    @raises(InvalidArgumentError)
    def test_notification_config_has_valid_keys(self):
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {
                'QueueConfiguration': [
                    {
                        'Id': '1',
                        'Arn': 'arn1',
                        'Events': ['s3:ObjectCreated:*'],
                    }
                ]
            }
        )

    @raises(InvalidArgumentError)
    def test_notification_config_arn_key_is_present(self):
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {
                'QueueConfigurations': [
                    {
                        'Id': '1',
                        'Events': ['s3:ObjectCreated:*'],
                    }
                ]
            }
        )

    @raises(InvalidArgumentError)
    def test_notification_config_id_key_is_string(self):
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {
                'QueueConfigurations': [
                    {
                        'Id': 1,
                        'Arn': 'abc',
                        'Events': ['s3:ObjectCreated:*'],
                    }
                ]
            }
        )

    @raises(InvalidArgumentError)
    def test_notification_config_events_key_is_present(self):
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {
                'QueueConfigurations': [
                    {
                        'Id': '1',
                        'Arn': 'arn1',
                    }
                ]
            }
        )

    @raises(InvalidArgumentError)
    def test_notification_config_event_values_are_valid(self):
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {
                'QueueConfigurations': [
                    {
                        'Id': '1',
                        'Arn': 'arn1',
                        'Events': ['object_created']
                    }
                ]
            }
        )

    @mock.patch('urllib3.PoolManager')
    def test_notification_config_id_key_is_optional(self, mock_connection):
        mock_server = MockConnection()
        mock_connection.return_value = mock_server
        mock_server.mock_add_request(
            MockResponse(
                'PUT',
                'https://localhost:9000/my-test-bucket/?notification=',
                {
                    'Content-Md5': 'f+TfVp/A4pNnI7S4S+MkFg==',
                    'Content-Length': '196',
                    'User-Agent': _DEFAULT_USER_AGENT,
                },
                200, content=""
            )
        )
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {
                'QueueConfigurations': [
                    {
                        'Arn': 'arn1',
                        'Events': ['s3:ObjectCreated:*'],
                    }
                ]
            }
        )

    @raises(InvalidArgumentError)
    def test_notification_config_has_valid_event_names(self):
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {
                'QueueConfigurations': [
                    {
                        'Id': '1',
                        'Arn': 'arn1',
                        'Events': ['object_created'],
                    }
                ]
            }
        )

    @raises(InvalidArgumentError)
    def test_notification_config_filterspec_is_valid_1(self):
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {
                'QueueConfigurations': [
                    {
                        'Id': '1',
                        'Arn': 'arn1',
                        'Events': ['s3:ObjectCreated:*'],
                        'Filter': []
                    }
                ]
            }
        )

    @raises(InvalidArgumentError)
    def test_notification_config_filterspec_is_valid_2(self):
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {
                'QueueConfigurations': [
                    {
                        'Id': '1',
                        'Arn': 'arn1',
                        'Events': ['s3:ObjectCreated:*'],
                        'Filter': {
                            'S3Key': {
                            }
                        }
                    }
                ]
            }
        )

    @raises(InvalidArgumentError)
    def test_notification_config_filterspec_is_valid_3(self):
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {
                'QueueConfigurations': [
                    {
                        'Id': '1',
                        'Arn': 'arn1',
                        'Events': ['s3:ObjectCreated:*'],
                        'Filter': {
                            'Key': {
                            }
                        }
                    }
                ]
            }
        )

    @raises(InvalidArgumentError)
    def test_notification_config_filterspec_is_valid_4(self):
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {
                'QueueConfigurations': [
                    {
                        'Id': '1',
                        'Arn': 'arn1',
                        'Events': ['s3:ObjectCreated:*'],
                        'Filter': {
                            'Key': {
                                'FilterRules': []
                            }
                        }
                    }
                ]
            }
        )

    @raises(InvalidArgumentError)
    def test_notification_config_filterspec_is_valid_5(self):
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {
                'QueueConfigurations': [
                    {
                        'Id': '1',
                        'Arn': 'arn1',
                        'Events': ['s3:ObjectCreated:*'],
                        'Filter': {
                            'Key': {
                                'FilterRules': [
                                    {
                                        'rule1': 'ab',
                                        'val1': 'abc'
                                    }
                                ]
                            }
                        }
                    }
                ]
            }
        )

    @raises(InvalidArgumentError)
    def test_notification_config_filterspec_is_valid_6(self):
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {
                'QueueConfigurations': [
                    {
                        'Id': '1',
                        'Arn': 'arn1',
                        'Events': ['s3:ObjectCreated:*'],
                        'Filter': {
                            'Key': {
                                'FilterRules': [
                                    {
                                        'Name': 'ab',
                                        'Value': 'abc'
                                    }
                                ]
                            }
                        }
                    }
                ]
            }
        )

    @mock.patch('urllib3.PoolManager')
    def test_notification_config_filterspec_is_valid_7(self, mock_connection):
        mock_server = MockConnection()
        mock_connection.return_value = mock_server
        mock_server.mock_add_request(
            MockResponse(
                'PUT',
                'https://localhost:9000/my-test-bucket/?notification=',
                {
                    'Content-Md5': 'k97dHBBUq9MR7ZViy7oUsw==',
                    'User-Agent': _DEFAULT_USER_AGENT,
                    'Content-Length': '300',
                },
                200, content=""
            )
        )
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {
                'QueueConfigurations': [
                    {
                        'Id': '1',
                        'Arn': 'arn1',
                        'Events': ['s3:ObjectCreated:*'],
                        'Filter': {
                            'Key': {
                                'FilterRules': [
                                    {
                                        'Name': 'prefix',
                                        'Value': 'abc'
                                    }
                                ]
                            }
                        }
                    }
                ]
            }
        )

    @mock.patch('urllib3.PoolManager')
    def test_notification_config_filterspec_is_valid_8(self, mock_connection):
        mock_server = MockConnection()
        mock_connection.return_value = mock_server
        mock_server.mock_add_request(
            MockResponse(
                'PUT',
                'https://localhost:9000/my-test-bucket/?notification=',
                {
                    'Content-Length': '300',
                    'Content-Md5': '2aIwAt1lAd5JShphHCD4GA==',
                    'User-Agent': _DEFAULT_USER_AGENT,
                },
                200, content=""
            )
        )
        client = Minio('localhost:9000')
        client.set_bucket_notification(
            'my-test-bucket',
            {
                'QueueConfigurations': [
                    {
                        'Id': '1',
                        'Arn': 'arn1',
                        'Events': ['s3:ObjectCreated:*'],
                        'Filter': {
                            'Key': {
                                'FilterRules': [
                                    {
                                        'Name': 'suffix',
                                        'Value': 'abc'
                                    }
                                ]
                            }
                        }
                    }
                ]
            }
        )



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage.
# Copyright (C) 2020 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Note: YOUR-ACCESSKEYID, YOUR-SECRETACCESSKEY and my-bucketname are
# dummy values, please replace them with original values.

from minio import Minio
from minio.error import ResponseError

client = Minio('s3.amazonaws.com',
               access_key='YOUR-ACCESSKEYID',
               secret_key='YOUR-SECRETACCESSKEY',
               secure=True)

try:
    # Get current policy of bucket 'my-bucketname'.
    print(client.get_bucket_encryption('my-bucketname'))
except ResponseError as err:
    print(err)



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C) 2015 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import TestCase
from nose.tools import eq_
from minio.helpers import get_s3_endpoint, is_valid_endpoint

class GetS3Endpoint(TestCase):
    def test_get_s3_endpoint(self):
        eq_('s3.amazonaws.com', get_s3_endpoint('us-east-1'))
        eq_('s3.amazonaws.com', get_s3_endpoint('foo'))
        eq_('s3-eu-west-1.amazonaws.com', get_s3_endpoint('eu-west-1'))
        eq_('s3.cn-north-1.amazonaws.com.cn', get_s3_endpoint('cn-north-1'))

    def test_is_valid_endpoint(self):
        eq_(True, is_valid_endpoint('s3.amazonaws.com'))
        eq_(True, is_valid_endpoint('s3.cn-north-1.amazonaws.com.cn'))
        eq_(True, is_valid_endpoint('minio_server:9000'))
        eq_(True, is_valid_endpoint('s3.server_1.amazonaws.com'))
        



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C)
# 2020 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import ABCMeta, abstractmethod
from datetime import datetime, timedelta

class Value(object):
    def __init__(self, access_key=None, secret_key=None, session_token=None):
        self.access_key = access_key
        self.secret_key = secret_key
        self.session_token = session_token

class Provider(object):

    __metaclass__ = ABCMeta

    @abstractmethod
    def retrieve(self):
        pass

    @abstractmethod
    def is_expired(self):
        pass

class Expiry(object):
    def __init__(self):
        self._expiration = None

    def set_expiration(self, expiration, time_delta=None):
        self._expiration = expiration
        if time_delta is not None:
            self._expiration = self._expiration + time_delta

    def is_expired(self):
        if self._expiration is None:
            return True
        return self._expiration < datetime.now()

class Credentials(object):
    def __init__(self, forceRefresh=True, provider=None):
        self._creds = None
        self._forceRefresh = forceRefresh
        self._provider = provider

    def get(self):
        if self.is_expired():
            try:
                creds = self._provider.retrieve()
            except:
                raise
            self._creds = creds
            self._forceRefresh = False
        return self._creds

    def expire(self):
        self._forceRefresh = True

    def is_expired(self):
        return self._forceRefresh or self._provider.is_expired()



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C) 2020 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# IamEc2MetaData will call AWS metadata service to retrieve credentials
#
# The default AWS metadata service can be found at:
#   -> 169.254.169.254/latest/meta-data/iam/security-credentials
#
# If you wish to retrieve credentials from a different place you can provide
# the 'endpoint' paramater to the IamEc2MetaData credentials object

from minio import Minio
from minio.credentials import IamEc2MetaData, Credentials

# Initialize Minio with IamEc2MetaData default credentials object
client = Minio('s3.amazonaws.com',
               credentials=Credentials(
                   provider=IamEc2MetaData()
               ))

# Initialize Minio with IamEc2MetaData custom
client = Minio('s3.amazonaws.com',
               credentials=Credentials(
                    provider=IamEc2MetaData(endpoint='custom.endpoint')
               ))



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C)
# 2020 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from unittest import TestCase

from minio.credentials.chain import Chain
from minio.credentials.credentials import Value
from minio.credentials.env_aws import EnvAWS
from minio.credentials.env_minio import EnvMinio
from nose.tools import eq_

class ChainProviderTest(TestCase):

    def test_chain_retrieve(self):
        # clear environment
        os.environ.clear()
        # prepare env for env_aws provider
        os.environ["AWS_ACCESS_KEY_ID"] = "access_aws"
        os.environ["AWS_SECRET_ACCESS_KEY"] = "secret_aws"
        os.environ["AWS_SESSION_TOKEN"] = "token_aws"
        # prepare env for env_minio
        os.environ['MINIO_ACCESS_KEY'] = "access_minio"
        os.environ["MINIO_SECRET_KEY"] = "secret_minio"
        # create chain provider with env_aws and env_minio providers
        chain = Chain(
            providers=[
                EnvAWS(),
                EnvMinio()
            ]
        )
        # retireve provider (env_aws) has priority
        creds = chain.retrieve()
        # assert provider credentials
        eq_(creds.access_key, "access_aws")
        eq_(creds.secret_key, "secret_aws")
        eq_(creds.session_token, "token_aws")
        # assert is_expired
        eq_(chain.is_expired(), False)
        

    def test_chain_is_expired(self):
        # clear environment
        os.environ.clear()
        # prepare env for env_aws
        os.environ["AWS_ACCESS_KEY_ID"] = "access_aws"
        os.environ["AWS_SECRET_ACCESS_KEY"] = "secret_aws"
        # create chain provider
        chain = Chain(
            providers=[EnvAWS()]
        )
        # is_expired should be True before retrieve()
        eq_(chain.is_expired(), True)
        # retieve single env_aws provider
        chain.retrieve()
        # is_expired should be False after retrieve()
        eq_(chain.is_expired(), False)

    def test_chain_with_no_provider(self):
        # clear environment
        os.environ.clear()  
        # create empty chain provider
        chain = Chain(
            providers=[]
        )
        # is_expired should be True before retrieve()
        eq_(chain.is_expired(), True)



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage,
# (C) 2018 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from nose.tools import eq_, raises
from unittest import TestCase

from minio.helpers import is_storageclass_header,is_amz_header,is_supported_header,amzprefix_user_metadata

class HeaderTests(TestCase):
    header_variants = {
        "content-type": [
            "content-type",
            "Content-Type",
            "CONTENT-TYPE",
            "cONTENT-tYPE",
            "cOntent-TypE",
            "CoNTENT-tYPe",
        ],
        "x-amz-meta-me": [
            "x-amz-meta-me",
            "X-Amz-Meta-Me",
            "X-AMZ-META-ME",
            "x-aMZ-mETA-mE",
        ],
        "cache-control": [
            "cache-control" ,
            "Cache-Control" ,
            "CACHE-CONTROL" ,
            "cACHE-cONTROL" ,
            "CacHe-conTrol" ,
        ],
        "content-disposition": [
            "content-disposition",
            "Content-Disposition",
            "CONTENT-DISPOSITION",
            "cONTENT-dISPOSITION",
            "conTent-disPositioN",
        ],
        "content-language": [
            "content-language",
            "Content-Language",
            "CONTENT-LANGUAGE",
            "conTent-Language",
        ],
        "x-amz-website-redirect-location": [
            "x-amz-website-redirect-location",
            "X-Amz-Website-Redirect-Location",
            "X-AMZ-WEBSITE-REDIRECT-LOCATION",
            "x-aMZ-wEBSITE-rEDIRECT-lOCATION",
        ],
        "x-amz-meta-status-code": [
            "x-amz-meta-status-code",
            "X-Amz-Meta-Status-Code",
            "X-AMZ-META-STATUS-CODE",
            "x-aMZ-mETA-sTATUS-cODE",
        ],
        "x-amz-server-side-encryption": [
            "x-amz-server-side-encryption",
            "X-Amz-Server-Side-Encryption",
            "X-AMZ-SERVER-SIDE-ENCRYPTION",
            "x-aMZ-sERVER-sIDE-eNCRYPTION",
        ],
        "x-amz-storage-class": [
            "x-amz-storage-class",
            "X-Amz-Storage-Class",
            "X-AMZ-STORAGE-CLASS",
            "x-aMZ-sTORAGE-cLASS",
        ],
    }

    def check_ok_header(self, check_fun, header):
        for header_variant in self.header_variants.get(header, [header]):
            eq_(check_fun(header_variant), True)

    def check_bad_header(self, check_fun, header):
        for header_variant in self.header_variants.get(header, [header]):
            eq_(check_fun(header_variant), False)

    def test_is_supported_header(self):
        self.check_ok_header(is_supported_header, "content-type")
        self.check_ok_header(is_supported_header, "cache-control")
        self.check_ok_header(is_supported_header, "content-disposition")
        self.check_ok_header(is_supported_header, "content-encoding")
        self.check_ok_header(is_supported_header, "content-language")
        self.check_ok_header(is_supported_header, "x-amz-website-redirect-location")

    def test_is_not_supported_header(self):
        self.check_bad_header(is_supported_header, "x-amz-meta-me")

    def test_is_amz_header(self):
        self.check_ok_header(is_amz_header, "x-amz-meta-status-code")
        self.check_ok_header(is_amz_header, "x-amz-server-side-encryption")

    def test_is_not_amz_header(self):
        self.check_bad_header(is_amz_header, "X_AMZ_META-VALUE")
        self.check_bad_header(is_amz_header, "content-type")

    def test_is_storageclass_header(self):
        self.check_ok_header(is_storageclass_header, "x-amz-storage-class")

    def test_is_not_storageclass_header(self):
        self.check_bad_header(is_storageclass_header, "x-amz-storage-classs")

    def test_amzprefix_user_metadata(self):
        metadata = {
                  'x-amz-meta-testing': 'values',
                  'x-amz-meta-setting': 'zombies',
                  'amz-meta-setting': 'zombiesddd',
                  'hhh':34,
                  'u_u': 'dd',
                  'y-fu-bar': 'zoo',
                  'Content-Type': 'application/csv',
                  'x-amz-storage-class': 'REDUCED_REDUNDANCY',
                  'content-language':'fr'
                  }
        m = amzprefix_user_metadata(metadata)
        self.assertTrue('Content-Type' in m)
        self.assertTrue('content-language' in m)

        self.assertTrue('X-Amz-Meta-hhh' in m)
        self.assertTrue('x-amz-storage-class' in m)
        self.assertTrue('X-Amz-Meta-amz-meta-setting' in m)



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C)
# 2020 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from unittest import TestCase

from minio.credentials.file_aws_credentials import FileAWSCredentials
from minio.credentials.credentials import Value
from nose.tools import eq_

class FileAWSCredentialsTest(TestCase):
    
    def test_file_aws(self):
        # clear environment
        os.environ.clear()
        # get provider
        provider = FileAWSCredentials('minio/credentials/credentials.sample')
        # is_expired should be True before retrieve
        eq_(provider.is_expired(), True)
        # retrieve credentials
        creds = provider.retrieve()
        expected_creds = Value(
            access_key='accessKey',
            secret_key='secret',
            session_token='token'
        )
        # assert credentials
        eq_(creds.access_key, expected_creds.access_key)
        eq_(creds.secret_key, expected_creds.secret_key)
        eq_(creds.session_token, expected_creds.session_token)
        # is_expired should be False before retrieve
        eq_(provider.is_expired(), False)

    def test_file_aws_from_env(self):
        # clear environment
        os.environ.clear()
        # set env with aws config file
        os.environ['AWS_SHARED_CREDENTIALS_FILE'] = 'minio/credentials/credentials.sample'
        # get provider
        provider = FileAWSCredentials()
        # is_expired should be True before retrieve
        eq_(provider.is_expired(), True)
        # retieve credentials
        creds = provider.retrieve()
        expected_creds = Value(
            access_key='accessKey',
            secret_key='secret',
            session_token='token'
        )
        eq_(creds.access_key, expected_creds.access_key)
        eq_(creds.secret_key, expected_creds.secret_key)
        eq_(creds.session_token, expected_creds.session_token)
        # is_expired should be False after retrieve
        eq_(provider.is_expired(), False)

    def test_file_aws_env_profile(self):
        # clear environment
        os.environ.clear()
        # set profile env
        os.environ['AWS_PROFILE'] = 'no_token'
        # get provider
        provider = FileAWSCredentials('minio/credentials/credentials.sample')
        # is_expired should be True before retrieve
        eq_(provider.is_expired(), True)
        # retrieve credentials
        creds = provider.retrieve()
        expected_creds = Value(
            access_key='accessKey',
            secret_key='secret',
            session_token=''
        )
        eq_(creds.access_key, expected_creds.access_key)
        eq_(creds.secret_key, expected_creds.secret_key)
        eq_(creds.session_token, expected_creds.session_token)
        # is_expired should be False after retrieve
        eq_(provider.is_expired(), False)

    def test_file_aws_arg_profile(self):
        # clear environment
        os.environ.clear()
        # get provider
        provider = FileAWSCredentials('minio/credentials/credentials.sample', 'no_token')
        # is_expired should be True before retrieve
        eq_(provider.is_expired(), True)
        # retrieve credentials
        creds = provider.retrieve()
        expected_creds = Value(
            access_key='accessKey',
            secret_key='secret',
            session_token=''
        )
        eq_(creds.access_key, expected_creds.access_key)
        eq_(creds.secret_key, expected_creds.secret_key)
        eq_(creds.session_token, expected_creds.session_token)
        # is_expired should be False after retieve
        eq_(provider.is_expired(), False)

    def test_file_aws_no_creds(self):
        # clear environment
        os.environ.clear()
        provider = FileAWSCredentials('minio/credentials/credentials.empty', 'no_token')
        creds = provider.retrieve()
        eq_(creds.access_key, None)
        eq_(creds.secret_key, None)
        eq_(creds.session_token, None)



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C) 2015 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
import pytz

from nose.tools import eq_
from unittest import TestCase
from datetime import datetime

from minio import Minio
from minio.api import _DEFAULT_USER_AGENT

from .minio_mocks import MockResponse, MockConnection

class ListBucketsTest(TestCase):
    @mock.patch('urllib3.PoolManager')
    def test_empty_list_buckets_works(self, mock_connection):
        mock_data = '<ListAllMyBucketsResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/"><Buckets>' \
                    '</Buckets><Owner><ID>minio</ID><DisplayName>minio</DisplayName></Owner></ListAllMyBucketsResult>'
        mock_server = MockConnection()
        mock_connection.return_value = mock_server
        mock_server.mock_add_request(MockResponse('GET', 'https://localhost:9000/',
                                                  {'User-Agent': _DEFAULT_USER_AGENT},
                                                  200, content=mock_data))
        client = Minio('localhost:9000')
        buckets = client.list_buckets()
        count = 0
        for bucket in buckets:
            count += 1
        eq_(0, count)

    @mock.patch('urllib3.PoolManager')
    def test_list_buckets_works(self, mock_connection):
        mock_data = '<ListAllMyBucketsResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/"><Buckets>' \
                    '<Bucket><Name>hello</Name><CreationDate>2015-06-22T23:07:43.240Z</CreationDate></Bucket><Bucket>' \
                    '<Name>world</Name><CreationDate>2015-06-22T23:07:56.766Z</CreationDate></Bucket>' \
                    '</Buckets><Owner><ID>minio</ID><DisplayName>minio</DisplayName></Owner></ListAllMyBucketsResult>'
        mock_server = MockConnection()
        mock_connection.return_value = mock_server
        mock_server.mock_add_request(MockResponse('GET', 'https://localhost:9000/',
                                                  {'User-Agent': _DEFAULT_USER_AGENT},
                                                  200, content=mock_data))
        client = Minio('localhost:9000')
        buckets = client.list_buckets()
        buckets_list = []
        count = 0
        for bucket in buckets:
            count += 1
            buckets_list.append(bucket)
        eq_(2, count)
        eq_('hello', buckets_list[0].name)
        eq_(datetime(2015, 6, 22, 23, 7, 43, 240000, pytz.utc), buckets_list[0].creation_date)
        eq_('world', buckets_list[1].name)
        eq_(datetime(2015, 6, 22, 23, 7, 56, 766000, pytz.utc), buckets_list[1].creation_date)



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C) 2015 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Note: YOUR-ACCESSKEYID, YOUR-SECRETACCESSKEY, my-testfile, my-bucketname and
# my-objectname are dummy values, please replace them with original values.

import os

from minio import Minio
from minio.error import ResponseError
from examples.progress import Progress

client = Minio('s3.amazonaws.com',
               access_key='YOUR-ACCESSKEYID',
               secret_key='YOUR-SECRETACCESSKEY')

# Put a file with default content-type.
try:
    with open('my-testfile', 'rb') as file_data:
        file_stat = os.stat('my-testfile')
        client.put_object('my-bucketname', 'my-objectname',
                          file_data, file_stat.st_size)
except ResponseError as err:
    print(err)

# Put a file with 'application/csv'
try:
    with open('my-testfile.csv', 'rb') as file_data:
        file_stat = os.stat('my-testfile.csv')
        client.put_object('my-bucketname', 'my-objectname', file_data,
                          file_stat.st_size, content_type='application/csv')
except ResponseError as err:
    print(err)

# Put a file with progress.
progress = Progress()
try:
    with open('my-testfile', 'rb') as file_data:
        file_stat = os.stat('my-testfile')
        client.put_object('my-bucketname', 'my-objectname',
                          file_data, file_stat.st_size, progress=progress)
except ResponseError as err:
    print(err)



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C)
# 2015, 2016 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
minio.post_policy
~~~~~~~~~~~~~~~

This module contains :class:`PostPolicy <PostPolicy>` implementation.

:copyright: (c) 2015 by MinIO, Inc.
:license: Apache 2.0, see LICENSE for more details.

"""

import base64
import json
import datetime

from .helpers import (is_non_empty_string, is_valid_bucket_name)
from .error import InvalidArgumentError


# Policy explanation:
# http://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-HTTPPOSTConstructPolicy.html
class PostPolicy(object):
    """
    A :class:`PostPolicy <PostPolicy>` object for constructing
       Amazon S3 POST policy JSON string.
    """
    def __init__(self):
        self._expiration = None
        self._content_length_range = tuple()
        # publicly accessible
        self.policies = []
        self.form_data = dict()
        self.bucket_name = ''
        self.key = ''

    def set_expires(self, time):
        """
        Set expiration time :class:`datetime.datetime`.

        :param time: set expiration :class:`datetime.datetime`.
        """
        if time.toordinal() < 1:
            ValueError()
        self._expiration = time

    def set_key(self, key):
        """
        Set key policy condition.

        :param key: set key name.
        """
        is_non_empty_string(key)

        self.policies.append(('eq', '$key', key))
        self.form_data['key'] = key
        self.key = key

    def set_key_startswith(self, key_startswith):
        """
        Set key startswith policy condition.

        :param key_startswith: set key prefix name.
        """
        is_non_empty_string(key_startswith)

        self.policies.append(('starts-with', '$key', key_startswith))
        self.form_data['key'] = key_startswith

    def set_bucket_name(self, bucket_name):
        """
        Set bucket name policy condition.

        :param bucket_name: set bucket name.
        """
        is_valid_bucket_name(bucket_name, False)

        self.policies.append(('eq', '$bucket', bucket_name))
        self.form_data['bucket'] = bucket_name
        self.bucket_name = bucket_name

    def set_content_type(self, content_type):
        """
        Set content-type policy condition.

        :param content_type: set content type name.
        """
        self.policies.append(('eq', '$Content-Type', content_type))
        self.form_data['Content-Type'] = content_type

    def set_content_length_range(self, min_length, max_length):
        """
        Set content length range policy condition.
           Raise :exc:`ValueError` for invalid inputs.

        :param min_length: Minimum length limit for content size.
        :param max_length: Maximum length limit for content size.
        """
        err_msg = ('Min-length ({}) must be <= Max-length ({}), '
                   'and they must be non-negative.').format(
                       min_length, max_length
                   )
        if min_length > max_length or min_length < 0 or max_length < 0:
            raise ValueError(err_msg)

        self._content_length_range = (min_length, max_length)

    def append_policy(self, condition, target, value):
        self.policies.append([condition, target, value])

    def _marshal_json(self, extras=()):
        """
        Marshal various policies into json str/bytes.
        """
        policies = self.policies[:]
        policies.extend(extras)
        if self._content_length_range:
            policies.append(['content-length-range'] +
                            list(self._content_length_range))

        policy_stmt = {
            "expiration": self._expiration.strftime(
                "%Y-%m-%dT%H:%M:%S.000Z"),
        }

        if len(policies) > 0:
            policy_stmt["conditions"] = policies

        return json.dumps(policy_stmt)

    def base64(self, extras=()):
        """
        Encode json into base64.
        """
        s = self._marshal_json(extras=extras)
        s_bytes = s if isinstance(s, bytes) else s.encode('utf-8')
        b64enc = base64.b64encode(s_bytes)
        return b64enc.decode('utf-8') if isinstance(b64enc, bytes) else b64enc

    def is_valid(self):
        """
        Validate for required parameters.
        """
        if not isinstance(self._expiration, datetime.datetime):
            raise InvalidArgumentError('Expiration datetime must be specified.')

        if 'key' not in self.form_data:
            raise InvalidArgumentError('object key must be specified.')

        if 'bucket' not in self.form_data:
            raise InvalidArgumentError('bucket name must be specified.')



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C)
# 2020 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os, json, urllib3, datetime
from .credentials import Provider, Value, Expiry
from minio.error import ResponseError

class IamEc2MetaData(Provider):

    iam_security_creds_path = '/latest/meta-data/iam/security-credentials'

    default_expiry_window = datetime.timedelta(minutes=5)

    def __init__(self, endpoint=None):
        super(Provider, self).__init__()
        if endpoint == "" or endpoint is None:
            endpoint = "http://169.254.169.254"
        self._endpoint = endpoint
        self._expiry = Expiry()
        self._http_client = urllib3.PoolManager(
                retries=urllib3.Retry(
                    total=5,
                    backoff_factor=0.2,
                    status_forcelist=[500, 502, 503, 504]
                    )
                )

    def request_cred_list(self):
        url = self._endpoint + self.iam_security_creds_path
        try:
            res = self._http_client.urlopen('GET', url)
            if res.status != 200:
                return None
        except:
            return None
        creds = res.data
        return creds.decode("utf-8").split('\n')

    def request_cred(self, creds_name):
        url = self._endpoint + self.iam_security_creds_path + "/" + creds_name
        res = self._http_client.urlopen('GET', url)
        if res.status != 200:
            raise ResponseError(res, 'GET')

        data = json.loads(res.data)
        if data['Code'] != 'Success':
            raise ResponseError(res)

        return data

    def retrieve(self):
        role_names = self.request_cred_list()
        if not role_names:
            return Value()

        creds_name = role_names[0]
        role_creds = self.request_cred(creds_name)
        expiration = datetime.datetime.strptime(role_creds['Expiration'], '%Y-%m-%dT%H:%M:%SZ')
        self._expiry.set_expiration(expiration, self.default_expiry_window)

        return Value(
            access_key=role_creds['AccessKeyId'],
            secret_key=role_creds['SecretAccessKey'],
            session_token=role_creds['Token']
        )

    def is_expired(self):
        return self._expiry.is_expired()



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C)
# 2020 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os

from .credentials import Provider, Value

class EnvMinio(Provider):
    def __init__(self, retrieved=False):
        super(Provider, self).__init__()
        self._retrieved = retrieved

    def retrieve(self):
        self._retrieved = False

        id = os.environ.get('MINIO_ACCESS_KEY')

        secret = os.environ.get('MINIO_SECRET_KEY')

        self._retrieved = True

        return Value(
            access_key=id,
            secret_key=secret
        )

    def is_expired(self):
        return not self._retrieved



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C)
# 2020 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .credentials import Provider, Value

class Chain(Provider):
    def __init__(self, providers):
        super(Provider, self).__init__()
        self._providers = providers
        self._current = None

    def retrieve(self):
        for provider in self._providers:
            creds = provider.retrieve()
            if ((creds.access_key is None or creds.access_key == "") and
                (creds.secret_key is None or creds.secret_key == "")):
                continue
            self._current = provider
            return creds
        self._current = None
        return Value()

    def is_expired(self):
        if self._current == None:
            return True
        return self._current.is_expired()



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage.
# Copyright (C) 2016 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Note: YOUR-ACCESSKEYID, YOUR-SECRETACCESSKEY and my-bucketname are
# dummy values, please replace them with original values.

from minio import Minio
from minio.error import ResponseError
import json

client = Minio('s3.amazonaws.com',
               access_key='YOUR-ACCESSKEYID',
               secret_key='YOUR-SECRETACCESSKEY')

# Make a new bucket
try:
    # Set bucket policy to read-only for bucket 'my-bucketname'
    policy_read_only = {
        "Version":"2012-10-17",
        "Statement":[
            {
            "Sid":"",
            "Effect":"Allow",
            "Principal":{"AWS":"*"},
            "Action":"s3:GetBucketLocation",
            "Resource":"arn:aws:s3:::my-bucketname"
            },
            {
            "Sid":"",
            "Effect":"Allow",
            "Principal":{"AWS":"*"},
            "Action":"s3:ListBucket",
            "Resource":"arn:aws:s3:::my-bucketname"
            },
            {
            "Sid":"",
            "Effect":"Allow",
            "Principal":{"AWS":"*"},
            "Action":"s3:GetObject",
            "Resource":"arn:aws:s3:::my-bucketname/*"
            }
        ]
    }
    client.set_bucket_policy('my-bucketname', json.dumps(policy_read_only))

    # Set bucket policy to read-write for bucket 'my-bucketname'
    policy_read_write = {
        "Version": "2012-10-17",
        "Statement": [
            {
                "Action": ["s3:GetBucketLocation"],
                "Sid": "",
                "Resource": ["arn:aws:s3:::my-bucketname"],
                "Effect": "Allow",
                "Principal": {"AWS": "*"}
            },
            {
                "Action": ["s3:ListBucket"],
                "Sid": "",
                "Resource": ["arn:aws:s3:::my-bucketname"],
                "Effect": "Allow",
                "Principal": {"AWS": "*"}
            },
            {
                "Action": ["s3:ListBucketMultipartUploads"],
                "Sid": "",
                "Resource": ["arn:aws:s3:::my-bucketname"],
                "Effect": "Allow",
                "Principal": {"AWS": "*"}
            },
            {
                "Action": ["s3:ListMultipartUploadParts",
                            "s3:GetObject",
                            "s3:AbortMultipartUpload",
                            "s3:DeleteObject",
                            "s3:PutObject"],
                "Sid": "",
                "Resource": ["arn:aws:s3:::my-bucketname/*"],
                "Effect": "Allow",
                "Principal": {"AWS": "*"}
            }
        ]
    }
    client.set_bucket_policy('my-bucketname', json.dumps(policy_read_write))

    # Set bucket policy to write-only for bucket 'my-bucketname'
    policy_write_only = {
        "Version":"2012-10-17",
        "Statement":[
            {
                "Sid":"",
                "Effect":"Allow",
                "Principal":{"AWS":"*"},
                "Action":"s3:GetBucketLocation",
                "Resource":"arn:aws:s3:::my-bucketname"
            },
            {"Sid":"",
            "Effect":"Allow",
            "Principal":{"AWS":"*"},
            "Action":"s3:ListBucketMultipartUploads",
            "Resource":"arn:aws:s3:::my-bucketname"
            },
            {
                "Sid":"",
                "Effect":"Allow",
                "Principal":{"AWS":"*"},
                "Action":[
                    "s3:ListMultipartUploadParts",
                    "s3:AbortMultipartUpload",
                    "s3:DeleteObject",
                    "s3:PutObject"],
                "Resource":"arn:aws:s3:::my-bucketname/*"
            }
        ]
    }
    client.set_bucket_policy('my-bucketname', json.dumps(policy_write_only))

except ResponseError as err:
    print(err)



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C) 2015 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
minio.definitions
~~~~~~~~~~~~~~~

This module contains the primary objects that power MinIO.

:copyright: (c) 2015 by MinIO, Inc.
:license: Apache 2.0, see LICENSE for more details.

"""


class Bucket(object):
    """
    A bucket metadata :class:`Bucket <Bucket>`.

    :param name: Bucket name.
    :param created: Bucket creation date.
    """
    def __init__(self, name, created):
        self.name = name
        self.creation_date = created

    def __str__(self):
        return '<Bucket: {0} {1}>'.format(self.name, self.creation_date)


class Object(object):
    """
    A object metadata :class:`Object <Object>`.

    :param bucket_name: Bucket name.
    :param object_name: Object name.
    :param last_modified: Object when it was last modified on server.
    :param etag: ETag saved on server for the object_name.
    :param size: Size of the object on server.
    :param content_type: Optional parameter indicating content type.
    :param is_dir: Optional parameter differentiating object prefixes.
    :param metadata: Optional parameter contains all the custom metadata.
    """
    def __init__(self, bucket_name, object_name, last_modified, etag, size,
                 content_type=None, is_dir=False, metadata=None):
        self.bucket_name = bucket_name
        self.object_name = object_name
        self.last_modified = last_modified
        self.etag = etag
        self.size = size
        self.content_type = content_type
        self.is_dir = is_dir
        self.metadata = metadata

    def __str__(self):
        string_format = '<Object: bucket_name: {0} object_name: {1}' \
                        ' last_modified: {2} etag: {3} size: {4}' \
                        ' content_type: {5}, is_dir: {6}, metadata: {7}>'
        return string_format.format(self.bucket_name,
                                    self.object_name.encode('utf-8'),
                                    self.last_modified,
                                    self.etag, self.size,
                                    self.content_type,
                                    self.is_dir,
                                    self.metadata)

class MultipartUploadResult(object):
    """
    A completed multipart upload metadata
         :class:`MultipartUploadResult <MultipartUploadResult>`.

    :param bucket_name: Bucket name.
    :param object_name: Object name.
    :param location: Object uploaded location.
    :param etag: Object final etag.
    """
    def __init__(self, bucket_name, object_name, location, etag):
        self.bucket_name = bucket_name
        self.object_name = object_name
        self.location = location
        self.etag = etag

    def __str__(self):
        string_format = ('<IncompleteUpload: bucket_name: {0}'
                         ' object_name: {1} location: {2} etag: {3}>')
        return string_format.format(self.bucket_name, self.object_name,
                                    self.location, self.etag)

class CopyObjectResult(object):
    """
    A complete copy object operation metadata.
         :class:`CopyObjectResult <CopyObjectResult>`.

    :param bucket_name: Bucket name.
    :param object_name: Object name.
    :param etag: ETag saved on the server computed for object_name.
    :param last_modified: Object when it was last modified on server.
    """
    def __init__(self, bucket_name, object_name, etag, last_modified):
        self.bucket_name = bucket_name
        self.object_name = object_name
        self.etag = etag
        self.last_modified = last_modified

    def __str__(self):
        string_format = ('<CopyObjectResult: bucket_name: {0}'
                         ' object_name: {1} etag: {2} last_modified: {3}>')
        return string_format.format(self.bucket_name, self.object_name,
                                    self.etag, self.last_modified)

class IncompleteUpload(object):
    """
    A partially uploaded object's metadata
         :class:`IncompleteUpload <IncompleteUpload>`.

    :param bucket_name: Bucket name.
    :param object_name: Object name.
    :param upload_id: Partially uploaded object's upload id.
    :param initiated: Date when the multipart was initiated.
    """
    def __init__(self, bucket_name, object_name, upload_id, initiated):
        self.bucket_name = bucket_name
        self.object_name = object_name
        self.upload_id = upload_id
        self.initiated = initiated
        self.size = 0

    def __str__(self):
        string_format = ('<IncompleteUpload: bucket_name: {0}'
                         ' object_name: {1} upload_id: {2}'
                         ' initiated:{3} size: {4}>')
        return string_format.format(self.bucket_name, self.object_name,
                                    self.upload_id, self.initiated, self.size)


class UploadPart(object):
    """
    A multipart upload part metadata :class:`UploadPart <UploadPart>`

    :param bucket_name: Bucket name.
    :param object_name: Object name.
    :param upload_id: Partially uploaded object's upload id.
    :param part_number: Part number of the part.
    :param etag: ETag of the part.
    :last_modified: Last modified time of the part.
    :size: Size of the part.
    """
    def __init__(self, bucket_name, object_name, upload_id, part_number, etag,
                 last_modified, size):
        self.bucket_name = bucket_name
        self.object_name = object_name
        self.upload_id = upload_id
        self.part_number = part_number
        self.etag = etag
        self.last_modified = last_modified
        self.size = size

    def __str__(self):
        string_format = '<UploadPart: bucket_name: {0} object_name: {1}' \
                        ' upload_id: {2} part_number: {3} etag: {4}' \
                        ' last_modified: {5} size: {6}>'
        return string_format.format(self.bucket_name,
                                    self.object_name,
                                    self.upload_id,
                                    self.part_number,
                                    self.etag,
                                    self.last_modified,
                                    self.size)



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C)
# 2020 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from minio.credentials.static import Static
from minio.credentials.credentials import Value
from unittest import TestCase
from nose.tools import eq_

class StaticTest(TestCase):
    def test_static_credentials(self):
        # get provider
        provider = Static(
            access_key='UXHW',
            secret_key='SECRET'
        )
        # static is_expired is always False
        eq_(provider.is_expired(), False)
        # retrieve creds
        creds = provider.retrieve()
        expected_creds = Value(
            access_key='UXHW',
            secret_key='SECRET'
        )
        eq_(creds.access_key, expected_creds.access_key)
        eq_(creds.secret_key, expected_creds.secret_key)
        # static is_expired is always False
        eq_(provider.is_expired(), False)

    def test_empty_static_credentials(self):
        # get provider
        provider = Static()
        # static is_expired is always False
        eq_(provider.is_expired(), False)
        # retrieve credentials
        creds = provider.retrieve()
        expected_creds = Value()
        eq_(creds.access_key, expected_creds.access_key)
        eq_(creds.secret_key, expected_creds.secret_key)
        eq_(creds.session_token, expected_creds.session_token)
        # static is_expired is always False
        eq_(provider.is_expired(), False)



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C) 2015 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Note: my-bucketname, my-objectname, YOUR-ACCESSKEYID, and YOUR-SECRETACCESSKEY
# are dummy values, please replace them with original values.

from datetime import datetime, timedelta

from minio import Minio
from minio import PostPolicy
from minio.error import ResponseError

post_policy = PostPolicy()
# set bucket name location for uploads.
post_policy.set_bucket_name('my-bucketname')
# set key prefix for all incoming uploads.
post_policy.set_key_startswith('my-objectname')
# set content length for incoming uploads.
post_policy.set_content_length_range(10, 1024)

# set expiry 10 days into future.
expires_date = datetime.utcnow()+timedelta(days=10)
post_policy.set_expires(expires_date)

client = Minio('s3.amazonaws.com',
               access_key='YOUR-ACCESSKEYID',
               secret_key='YOUR-SECRETACCESSKEY')

try:
    url_str, signed_form_data = client.presigned_post_policy(post_policy)
    curl_str = 'curl -X POST {0}'.format(url_str)
    curl_cmd = [curl_str]
    for field in signed_form_data:
        curl_cmd.append('-F {0}={1}'.format(field, signed_form_data[field]))

    # print curl command to upload files.
    curl_cmd.append('-F file=@<FILE>')
    print(' '.join(curl_cmd))
except ResponseError as err:
    print(err)



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage,
# (C) 2015, 2016 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from nose.tools import eq_
from unittest import TestCase

from minio import Minio
from minio.api import _DEFAULT_USER_AGENT

from .minio_mocks import MockResponse, MockConnection

class ListIncompleteUploadsTest(TestCase):
    @mock.patch('urllib3.PoolManager')
    def test_empty_list_uploads_test(self, mock_connection):
        mock_data = '''<?xml version="1.0"?>
                       <ListMultipartUploadsResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
                         <Bucket>golang</Bucket>
                         <KeyMarker/>
                         <UploadIdMarker/>
                         <NextKeyMarker/>
                         <NextUploadIdMarker/>
                         <EncodingType/>
                         <MaxUploads>1000</MaxUploads>
                         <IsTruncated>false</IsTruncated>
                         <Prefix/>
                         <Delimiter/>
                       </ListMultipartUploadsResult>
                    '''
        mock_server = MockConnection()
        mock_connection.return_value = mock_server
        mock_server.mock_add_request(
            MockResponse('GET',
                         'https://localhost:9000/bucket/?prefix=&uploads=',
                         {'User-Agent': _DEFAULT_USER_AGENT}, 200, content=mock_data))
        client = Minio('localhost:9000')
        upload_iter = client._list_incomplete_uploads('bucket', '', True, False)
        uploads = []
        for upload in upload_iter:
            uploads.append(upload)
        eq_(0, len(uploads))

    @mock.patch('urllib3.PoolManager')
    def test_list_uploads_works(self, mock_connection):
        mock_data = '''<?xml version="1.0"?>
                       <ListMultipartUploadsResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
                         <Bucket>golang</Bucket>
                         <KeyMarker/>
                         <UploadIdMarker/>
                         <NextKeyMarker>keymarker</NextKeyMarker>
                         <NextUploadIdMarker>uploadidmarker</NextUploadIdMarker>
                         <EncodingType/>
                         <MaxUploads>1000</MaxUploads>
                         <IsTruncated>false</IsTruncated>
                         <Upload>
                           <Key>go1.4.2</Key>
                           <UploadId>uploadid</UploadId>
                           <Initiator>
                             <ID/>
                             <DisplayName/>
                           </Initiator>
                           <Owner>
                             <ID/>
                             <DisplayName/>
                           </Owner>
                           <StorageClass/>
                           <Initiated>2015-05-30T14:43:35.349Z</Initiated>
                         </Upload>
                         <Upload>
                           <Key>go1.5.0</Key>
                           <UploadId>uploadid2</UploadId>
                           <Initiator>
                             <ID/>
                             <DisplayName/>
                           </Initiator>
                           <Owner>
                             <ID/>
                             <DisplayName/>
                           </Owner>
                           <StorageClass/>
                           <Initiated>2015-05-30T15:00:07.759Z</Initiated>
                         </Upload>
                         <Prefix/>
                         <Delimiter/>
                       </ListMultipartUploadsResult>
                    '''
        mock_server = MockConnection()
        mock_connection.return_value = mock_server
        mock_server.mock_add_request(
            MockResponse('GET',
                         'https://localhost:9000/bucket/?delimiter=%2F&prefix=&uploads=',
                         {'User-Agent': _DEFAULT_USER_AGENT},
                         200, content=mock_data))

        client = Minio('localhost:9000')
        upload_iter = client._list_incomplete_uploads('bucket', '', False, False)
        uploads = []
        for upload in upload_iter:
            uploads.append(upload)
        eq_(2, len(uploads))

    @mock.patch('urllib3.PoolManager')
    def test_list_multipart_uploads_works(self, mock_connection):
        mock_data1 = '''<?xml version="1.0"?>
                        <ListMultipartUploadsResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
                          <Bucket>golang</Bucket>
                          <KeyMarker/>
                          <UploadIdMarker/>
                          <NextKeyMarker>keymarker</NextKeyMarker>
                          <NextUploadIdMarker>uploadidmarker</NextUploadIdMarker>
                          <EncodingType/>
                          <MaxUploads>1000</MaxUploads>
                          <IsTruncated>true</IsTruncated>
                          <Upload>
                            <Key>go1.4.2</Key>
                            <UploadId>uploadid</UploadId>
                            <Initiator>
                              <ID/>
                              <DisplayName/>
                            </Initiator>
                            <Owner>
                              <ID/>
                              <DisplayName/>
                            </Owner>
                            <StorageClass/>
                            <Initiated>2015-05-30T14:43:35.349Z</Initiated>
                          </Upload>
                          <Upload>
                            <Key>go1.5.0</Key>
                            <UploadId>uploadid2</UploadId>
                            <Initiator>
                              <ID/>
                              <DisplayName/>
                            </Initiator>
                            <Owner>
                              <ID/>
                              <DisplayName/>
                            </Owner>
                            <StorageClass/>
                            <Initiated>2015-05-30T15:00:07.759Z</Initiated>
                          </Upload>
                          <Prefix/>
                          <Delimiter/>
                        </ListMultipartUploadsResult>
                     '''
        mock_data2 = '''<?xml version="1.0"?>
                        <ListMultipartUploadsResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
                          <Bucket>golang</Bucket>
                          <KeyMarker/>
                          <UploadIdMarker/>
                          <NextKeyMarker/>
                          <NextUploadIdMarker/>
                          <EncodingType/>
                          <MaxUploads>1000</MaxUploads>
                          <IsTruncated>false</IsTruncated>
                          <Upload>
                            <Key>go1.4.2</Key>
                            <UploadId>uploadid</UploadId>
                            <Initiator>
                              <ID/>
                              <DisplayName/>
                            </Initiator>
                            <Owner>
                              <ID/>
                              <DisplayName/>
                            </Owner>
                            <StorageClass/>
                            <Initiated>2015-05-30T14:43:35.349Z</Initiated>
                          </Upload>
                          <Upload>
                            <Key>go1.5.0</Key>
                            <UploadId>uploadid2</UploadId>
                            <Initiator>
                              <ID/>
                              <DisplayName/>
                            </Initiator>
                            <Owner>
                              <ID/>
                              <DisplayName/>
                            </Owner>
                            <StorageClass/>
                            <Initiated>2015-05-30T15:00:07.759Z</Initiated>
                          </Upload>
                          <Prefix/>
                          <Delimiter/>
                        </ListMultipartUploadsResult>
                     '''
        mock_server = MockConnection()
        mock_connection.return_value = mock_server
        mock_server.mock_add_request(
            MockResponse('GET',
                         'https://localhost:9000/bucket/?prefix=&uploads=',
                         {'User-Agent': _DEFAULT_USER_AGENT}, 200, content=mock_data1))

        client = Minio('localhost:9000')
        upload_iter = client._list_incomplete_uploads('bucket', '', True, False)
        uploads = []
        for upload in upload_iter:
            mock_server.mock_add_request(MockResponse('GET',
                                                      'https://localhost:9000/bucket/?'
                                                      'key-marker=keymarker&'
                                                      'prefix=&'
                                                      'upload-id-marker=uploadidmarker&uploads=',
                                                      {'User-Agent': _DEFAULT_USER_AGENT}, 200, content=mock_data2))
            uploads.append(upload)

        eq_(4, len(uploads))



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C)
# 2019 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
minio.select.helpers
~~~~~~~~~~~~~~~

This module implements the helper functions for SelectObject responses.

:copyright: (c) 2019 by MinIO, Inc.
:license: Apache 2.0, see LICENSE for more details.

"""

import codecs
from binascii import crc32

SQL = 'SQL'  # Value for ExpressionType
EVENT_RECORDS = 'Records'  # Event Type is Records
EVENT_PROGRESS = 'Progress'  # Event Type Progress
EVENT_STATS = 'Stats'  # Event Type Stats
EVENT_CONT = 'Cont' # Event Type continue
EVENT_END = 'End'  # Event Type is End
EVENT_CONTENT_TYPE = "text/xml" # Event content xml type
EVENT = 'event'  # Message Type is event
ERROR = 'error'  # Message Type is error

def calculate_crc(value):
    '''
    Returns the CRC using crc32
    '''
    return crc32(value) & 0xffffffff

def validate_crc(current_value, expected_value):
    '''
    Validate through CRC check
    '''
    crc_current = calculate_crc(current_value)
    crc_expected = byte_int(expected_value)
    if crc_current == crc_expected:
        return True
    return False

def byte_int(data_bytes):
    '''
    Convert bytes to big-endian integer
    '''
    return int(codecs.encode(data_bytes, 'hex'), 16)



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C)
# 2020 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import configparser

from .credentials import Provider, Value

class FileAWSCredentials(Provider):
    def __init__(self, filename=None, profile=None, retrieved=False):
        super(Provider, self).__init__()
        self._filename = filename
        self._profile = profile
        self._retrieved = retrieved

    def retrieve(self):

        if self._filename == "" or self._filename is None:
            self._filename = os.environ.get('AWS_SHARED_CREDENTIALS_FILE')
            if self._filename == "" or self._filename is None:
                home_dir = os.environ.get('HOME')
                self._filename = os.path.join(home_dir, '.aws', 'credentials')

        if self._profile == "" or self._profile is None:
            self._profile = os.environ.get('AWS_PROFILE')
            if self._profile == "" or self._profile is None:
                self._profile = 'default'

        self._retrieved = False
        ini_profile = configparser.ConfigParser()
        ini_profile.read(self._filename)
        access_key = secret = session_token = ''
        try:
            access_key = ini_profile.get(self._profile, 'aws_access_key_id')
            secret = ini_profile.get(self._profile, 'aws_secret_access_key')
            session_token = ini_profile.get(self._profile, 'aws_session_token')
        except:
            pass

        if access_key == '' or secret == '':
            return Value()

        self._retrieved = True
        return Value(
            access_key=access_key,
            secret_key=secret,
            session_token=session_token)

    def is_expired(self):
        return not self._retrieved



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C)
# 2020 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import configparser
import os, json
import sys

from .credentials import Provider, Value

class FileMinioClient(Provider):
    def __init__(self, filename=None, alias=None, retrieved=False):
        super(Provider, self).__init__()
        self._filename = filename
        self._alias = alias
        self._retrieved = retrieved

    def retrieve(self):
        if self._filename == "" or self._filename is None:
            home_dir = os.environ.get('HOME')
            self._filename = os.path.join(home_dir, '.mc', 'config.json')
            if sys.platform == 'win32':
                self._filename = os.path.join(home_dir, 'mc', 'config.json')
        if self._alias == "" or self._alias is None:
            self._alias = os.environ.get('MINIO_ALIAS')
            if self._alias == "" or self._alias is None:
                self._alias = "s3"

        self._retrieved = False

        config = open(self._filename, 'r')
        doc = json.load(config)
        creds = doc['hosts'][self._alias]

        access_key = creds['accessKey']
        secret_key = creds['secretKey']

        self._retrieved = True

        return Value(
            access_key=access_key,
            secret_key=secret_key
        )

    def is_expired(self):
        return not self._retrieved



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C)
# 2020 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from .credentials import Provider, Value

class EnvAWS(Provider):
    def __init__(self, retrieved=False):
        super(Provider, self).__init__()
        self.retrieved = retrieved

    def retrieve(self):
        self.retrieved = False

        access_key = os.environ.get('AWS_ACCESS_KEY_ID')
        if access_key == "":
            access_key = os.environ.get('AWS_ACCESS_KEY')

        secret = os.environ.get('AWS_SECRET_ACCESS_KEY')
        if secret == "":
            secret = os.environ.get('AWS_SECRET_KEY')

        self.retrieved = True
        return Value(
            access_key=access_key,
            secret_key=secret,
            session_token=os.environ.get('AWS_SESSION_TOKEN')
        )

    def is_expired(self):
        return not self.retrieved



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C) 2015 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import hashlib
import hmac
from unittest import TestCase
from datetime import datetime

from nose.tools import eq_, raises
import pytz as pytz

from minio.signer import (generate_canonical_request, generate_string_to_sign,
                          generate_signing_key, generate_authorization_header,
                          presign_v4, sign_v4)
from minio.error import InvalidArgumentError
from minio.compat import urlsplit, urlencode, queryencode
from minio.fold_case_dict import FoldCaseDict
from minio.credentials import Credentials, Static
from minio.helpers import get_target_url

empty_hash = 'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'
dt = datetime(2015, 6, 20, 1, 2, 3, 0, pytz.utc)

class CanonicalRequestTest(TestCase):
    def test_simple_request(self):
        url = urlsplit('http://localhost:9000/hello')
        expected_signed_headers = ['x-amz-content-sha256', 'x-amz-date']
        expected_request_array = ['PUT', '/hello', '',
                                  'x-amz-content-sha256:' +
                                  empty_hash, 'x-amz-date:dateString',
                                  '', ';'.join(expected_signed_headers),
                                  empty_hash]
        headers_to_sign = FoldCaseDict({'X-Amz-Date': 'dateString',
                                        'X-Amz-Content-Sha256': empty_hash})

        expected_request = '\n'.join(expected_request_array)
        actual_request = generate_canonical_request('PUT',
                                                    url,
                                                    headers_to_sign,
                                                    expected_signed_headers,
                                                    empty_hash)

        eq_(expected_request, actual_request)

    def test_request_with_query(self):
        url = urlsplit('http://localhost:9000/hello?c=d&e=f&a=b')
        expected_signed_headers = ['x-amz-content-sha256', 'x-amz-date']
        expected_request_array = ['PUT', '/hello', 'c=d&e=f&a=b',
                                  'x-amz-content-sha256:' + empty_hash,
                                  'x-amz-date:dateString',
                                  '', ';'.join(expected_signed_headers),
                                  empty_hash]

        expected_request = '\n'.join(expected_request_array)

        headers_to_sign = FoldCaseDict({'X-Amz-Date': 'dateString',
                                        'X-Amz-Content-Sha256': empty_hash})

        actual_request = generate_canonical_request('PUT',
                                                    url,
                                                    headers_to_sign,
                                                    expected_signed_headers,
                                                    empty_hash)

        eq_(expected_request, actual_request)


class StringToSignTest(TestCase):
    def test_signing_key(self):
        expected_signing_key_list = ['AWS4-HMAC-SHA256', '20150620T010203Z',
                                     '20150620/us-east-1/s3/aws4_request',
                                     'b93e86965c269a0dfef37a8bec231ef8acf8cdb101a64eb700a46c452c1ad233']

        actual_signing_key = generate_string_to_sign(dt, 'us-east-1', 'request_hash')
        eq_('\n'.join(expected_signing_key_list), actual_signing_key)


class SigningKeyTest(TestCase):
    def test_generate_signing_key(self):
        key1_string = 'AWS4' + 'S3CR3T'
        key1 = key1_string.encode('utf-8')
        key2 = hmac.new(key1, '20150620'.encode('utf-8'), hashlib.sha256).digest()
        key3 = hmac.new(key2, 'region'.encode('utf-8'), hashlib.sha256).digest()
        key4 = hmac.new(key3, 's3'.encode('utf-8'), hashlib.sha256).digest()
        expected_result = hmac.new(key4, 'aws4_request'.encode('utf-8'), hashlib.sha256).digest()

        actual_result = generate_signing_key(dt, 'region', 'S3CR3T')

        eq_(expected_result, actual_result)


class AuthorizationHeaderTest(TestCase):
    def test_generate_authentication_header(self):
        expected_authorization_header = 'AWS4-HMAC-SHA256 Credential=public_key/20150620/region/s3/aws4_request, ' \
                                        'SignedHeaders=host;X-Amz-Content-Sha256;X-Amz-Date, Signature=signed_request'
        actual_authorization_header = generate_authorization_header('public_key', dt, 'region',
                                                                    ['host', 'X-Amz-Content-Sha256', 'X-Amz-Date'],
                                                                    'signed_request')
        eq_(expected_authorization_header, actual_authorization_header)

class PresignURLTest(TestCase):
    @raises(InvalidArgumentError)
    def test_presigned_no_access_key(self):
        credentials = Credentials(
            provider=Static()
        )
        presign_v4('GET', 'http://localhost:9000/hello', credentials, None)

    @raises(InvalidArgumentError)
    def test_presigned_invalid_expires(self):
        credentials = Credentials(
            provider=Static()
        )
        presign_v4('GET', 'http://localhost:9000/hello', credentials, region=None, headers={}, expires=0)

class SignV4Test(TestCase):
    def test_signv4(self):
        # Construct target url.
        credentials = Credentials(
            provider=Static(
                access_key='minio',
                secret_key='minio123'
            )
        )
        url = get_target_url('http://localhost:9000', bucket_name='testbucket',
             object_name='~testobject', bucket_region='us-east-1', query={'partID': '1', 'uploadID': '~abcd'})
        hdrs = sign_v4('PUT', url, 'us-east-1', credentials=credentials, request_datetime=dt)
        eq_(hdrs['Authorization'], 'AWS4-HMAC-SHA256 Credential=minio/20150620/us-east-1/s3/aws4_request, SignedHeaders=host;x-amz-content-sha256;x-amz-date, Signature=a2f4546f647981732bd90dfa5a7599c44dca92f44bea48ecc7565df06032c25b')

class UnicodeEncodeTest(TestCase):
    def test_unicode_urlencode(self):
        eq_(urlencode('/test/123/汉字'), '/test/123/%E6%B1%89%E5%AD%97')

    def test_unicode_queryencode(self):
        eq_(queryencode('/test/123/汉字'), '%2Ftest%2F123%2F%E6%B1%89%E5%AD%97')

    def test_unicode_urlencode_u(self):
        eq_(urlencode(u'/test/123/汉字'), '/test/123/%E6%B1%89%E5%AD%97')

    def test_unicode_queryencode_u(self):
        eq_(queryencode(u'/test/123/汉字'), '%2Ftest%2F123%2F%E6%B1%89%E5%AD%97')

    def test_unicode_urlencode_b(self):
        eq_(urlencode(b'/test/123/\xe6\xb1\x89\xe5\xad\x97'), '/test/123/%E6%B1%89%E5%AD%97')

    def test_unicode_queryencode_b(self):
        eq_(queryencode(b'/test/123/\xe6\xb1\x89\xe5\xad\x97'), '%2Ftest%2F123%2F%E6%B1%89%E5%AD%97')



# -*- coding: utf-8 -*-
# MinIO Python Library for Amazon S3 Compatible Cloud Storage, (C) 2020 MinIO, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# A Chain credentials provider, provides a way of chaining multiple providers together
# and will pick the first available using priority order of the 'providers' list

from minio.credentials import Chain, EnvAWS, EnvMinio, IamEc2MetaData

client = Minio('s3.amazonaws.com',
               credentials=Credentials(
                   provider=Chain(
                        providers=[
                            IamEc2MetaData(),
                            EnvAWS(),
                            EnvMinio()
                        ]
                   )
               ))
